import time
import subprocess
import pytest
import sqlite3

from common.TimeEntry import TimeEntry
from common.TimeSheet import TimeSheet
from common.DBHolder import DB

@pytest.fixture(scope="session")
def db():
    db_obj = DB("timmy-time.sqlite")
    yield db_obj
    # db_obj.delete()

def test_create_db(db):
    """
    the database object must be created when the object is created and no file is present
    """
    timesheet = TimeSheet(db.path)
    if not db.exists:
        pytest.fail(f"Module failed to create {db_path}")

@pytest.mark.parametrize('table_name', ["time_entry"])
def test_check_db_tables(db, table_name):
    """
    The database must check which tables are present.
    """
    timesheet = TimeSheet(db.path)

    cmd = """
    SELECT name FROM sqlite_master WHERE type='table' AND name=?;
    """
    db.cursor.execute(cmd, (table_name, ))
    result = db.cursor.fetchone()
    if not result:
        pytest.fail(f"Table {table_name} wasnt created...")

@pytest.mark.parametrize("table_name", ["time_entry"])
@pytest.mark.parametrize("table_field, field_value", [
    ("description", "abc"),
    ("category","abc"),
    ("client", "test")
])
@pytest.mark.parametrize("start_date, start_time, end_date, end_time", [
    ("today","lunch", "tomorrow","lunch")
])
def test_create_db_entry(db, table_name, start_date, start_time, end_date, end_time, table_field, field_value):

    timesheet = TimeSheet(db.path)

    time_entry = TimeEntry(start_date, start_time, end_date, end_time)
    if (table_field == "description"):
        getattr(time_entry, f"set_note")(field_value)
    else:
        getattr(time_entry, f"set_{table_field}")(field_value)

    timesheet.insert_time_entry(time_entry)

    result = db.find(table_name, table_field, field_value)
    assert len(result) == 1, "Failed to create an entry"


@pytest.mark.parametrize("start_date, start_time, end_date, end_time", [
    ("today", "lunch", "tomorrow", "lunch")
])
def test_get_db_entries(db, start_date, start_time, end_date, end_time):

    db.delete()

    timesheet = TimeSheet(db.path)

    for i in range(1,4):
        time_entry = TimeEntry(start_date, start_time)
        time_entry.set_note(f"line {i}")
        time_entry.set_duration(f"{i} hrs")

        timesheet.insert_time_entry(time_entry)

    entries = timesheet.get_time_entries()

    assert len(entries) == 3, f"more than 3 entreis are present - {len(entries)}"



import subprocess
import sqlite3

class DB:
    def __init__(self, db_path):
        self.path = db_path
        self.conn = sqlite3.connect(db_path)
        self.cursor = self.conn.cursor()

    @property
    def exists(self):
        res = subprocess.run(f"[ -f {self.path} ]", shell=True)

        if res.returncode == 0:
            return True
        elif res.returncode == 1:
            return False
        else:
            raise Exception(f"failed to check if {self.path} exists...")

    def find(self, table_name, key, value):
        """
        finds the row in the table w.r.t to the table, the key and a search value within the key

        Parameters:
            table_name (string): The table name within the db
            key (string): The key to search within the db
            value (arbitrary): A search value for the key

        Returns:
            result (row data): The table entry

        """
        cmd = f"SELECT * FROM {table_name} WHERE {key} = ?;"
        self.cursor.execute(cmd, (value,))
        result = self.cursor.fetchall()
        return result

    def delete(self):

        if not self.exists:
            return

        res = subprocess.run(f"rm {self.path}", shell=True)
        if res.returncode != 0:
            raise Exception(f"Failed to delete {self.path}")



#! ./venv/bin/python3
import typer
from typing_extensions import Annotated
from rich.console import Console
from rich.table import Table

from datetime import datetime, timedelta

from common.TimeEntry import TimeEntry
from common.TimeSheet import TimeSheet

app = typer.Typer()
console = Console()

@app.command()
def enter(
        start_date_string: Annotated[str, typer.Option("--start-date", "-sd", prompt=True)],
        start_time_string: Annotated[str, typer.Option("--start-time", "-st", prompt=True)],
        description: Annotated[str, typer.Option("--desc", "-d", prompt=True)],
        client: Annotated[str, typer.Option("--client","-cl", prompt=True)],
        category: Annotated[str, typer.Option("--category","-cat", prompt=True)],
        ):

    time_entry = TimeEntry(start_date_string, start_time_string)
    time_entry.set_note(description)
    time_entry.set_client(client)
    time_entry.set_category(category)

    task_end_type = typer.prompt("Use Duration (d) or End Time (t)")
    if task_end_type == "d":
        duration_string = typer.prompt("Duration of Work")
        time_entry.set_duration(duration_string)
    elif task_end_type == "t":
        end_date = typer.prompt("End Date")
        end_time = typer.prompt("End Time")
        time_entry.set_end_ts(end_date, end_time)

    start_date = time_entry.start_ts.strftime("%d-%m-%y")
    start_time = time_entry.start_ts.strftime("%H:%M")

    duration_hrs = str(round(time_entry.duration_sec / 3600, 5))

    end_date = time_entry.end_ts.strftime("%d-%m-%y")
    end_time = time_entry.end_ts.strftime("%H:%M")

    timesheet = TimeSheet()
    timesheet.insert_time_entry(time_entry)

@app.command()
def show():

    timesheet = TimeSheet()

    table = Table("start ts", "duration (hrs)", "end ts", "description", "client")
    for row in timesheet.get_time_entries():
        start_ts, duration_sec, end_ts, description, category, client = row
        duration_hrs = round(duration_sec/3600, 5)
        # 2024-05-28 12:00:00

        start_ts = datetime.strptime(start_ts, '%Y-%m-%d %H:%M:%S').strftime("%d/%m/%Y %H:%M")
        end_ts = datetime.strptime(end_ts, '%Y-%m-%d %H:%M:%S').strftime("%d/%m/%Y %H:%M")

        table.add_row(start_ts, str(duration_hrs), end_ts, description, client)
    console.print(table)


if __name__ == "__main__":
    app()



import os
from setuptools import setup, find_packages

def get_install_requires() -> list[str]:
    """
    Returns requirements.txt parsed to a list
    """
    fname = 'requirements.txt'
    targets = []

    with open(fname, 'r') as f:
        targets = f.read().splitlines()

    return targets

setup(
        name = "timmytimesheet",
        version = "0.1.3",
        description = "a basic timesheeting cli",
        author="jumper385 (Henry Chen)",
        install_package_data=True,
        install_requires = get_install_requires(),
        packages = find_packages(),
        keywords = ["timesheeting"],
        entry_points = {
            'console_scripts': [
                'timmy=timmy.main:app']},
            classifiers = [
                'Programming Language :: Python :: 3',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                ])




import os
import sqlite3
import subprocess
from datetime import datetime
from common.TimeEntry import TimeEntry

class TimeSheet:
    def __init__(self, sq3_path = None):
        self.sq3_path = sq3_path if sq3_path else self.find_db_path()
        self.sq3_conn = sqlite3.connect(self.sq3_path)
        self.sq3_cursor = self.sq3_conn.cursor()

        self._create_tables_if_not_exist()

    def insert_time_entry(self, time_entry):
        """
        Insert a time entry object and add it to the database

        Parameters:
            - time_entry (TimeEntry): The time entry object
        """

        cmd = """
        INSERT INTO time_entry (start_ts, duration_sec, end_ts, description, category, client)
        VALUES (?, ?, ?, ?, ?, ?)
        """

        self.sq3_cursor.execute(cmd, (
            time_entry.start_ts,
            time_entry.duration_sec,
            time_entry.end_ts,
            time_entry.note,
            time_entry.category,
            time_entry.client
            ))

        self.sq3_conn.commit()

    def find_db_path(self):
        """
        Find a database starting with `timmy-time_XXXXXX.sqlite` in the local directory
        If the database doesn't exist, it returns a database path to make

        Returns:
            sq3_path (string): The path to the sq3lite database path
        """
        cwd = self._get_cwd()
        db_name = self._find_db_in_dir(cwd)

        if not cwd:
            raise Exception("Failed to find cwd")

        if db_name:
            return f"{cwd}/{db_name}"
        else:
            return f"{cwd}/timmy-time.sqlite"

    @staticmethod
    def _find_db_in_dir(directory_path):
        """
        Looks in a directory for timmy-time_XXXXX.sqlite.

        Parameters:
            directory_path (string): The path of the directory to search in

        Returns:
            db_name (string): if db is present, it returns the db name. else it returns none
        """
        files = os.listdir(directory_path)

        if len(files) < 1:
            return None

        files_filtered = [*filter(lambda fn: "timmy-time" in fn, files)]
        if len(files_filtered) < 1:
            return None

        return files_filtered[0].strip()

    @staticmethod
    def _get_cwd():
        """
        Gets the current workign directory

        Returns:
            cwd_path (string): THe current working directory
        """
        res = subprocess.run("pwd", shell=True, capture_output=True, check=True)
        res.check_returncode()

        cwd = res.stdout.decode()

        return cwd.strip()

    def get_time_entries(self):
        """
        Get the time entries from the database

        NOTE: Unimplemented...
        Parameters:
            date_limit (string): the earliest date up until the current date
            row_limit (int): maximum number of rows from the latest row entry

        Returns:
            entry_collection ([time_entry]): a list of entires
                - start_ts, duration_sec, end_ts, description, category, client

        """
        cmd = "SELECT start_ts, duration_sec, end_ts, description, category, client FROM time_entry"

        self.sq3_cursor.execute(cmd)
        results = self.sq3_cursor.fetchall()

        return results

    def _create_tables_if_not_exist(self):
        """
        creates the following tables
        - time_entry: start_ts (datetime), duration_sec(int), end_ts(datetime), description(string), category(string), client(string)
        """
        cmd = """
        CREATE TABLE IF NOT EXISTS time_entry (
            start_ts DATETIME,
            duration_sec INTEGER,
            end_ts DATETIME,
            description TEXT,
            category TEXT,
            client TEXT
        );
        """
        self.sq3_cursor.execute(cmd)



from datetime import datetime, timedelta
import pytest
from common.TimeEntry import TimeEntry

@pytest.mark.parametrize("date_entry, intended_date", [
    ('yesterday', datetime.now() - timedelta(days=1)),
    ('today', datetime.now()),
    ('tomorrow', datetime.now() + timedelta(days=1)),
    ('last week', datetime.now() - timedelta(days=7)),
    ('last fortnight', datetime.now() - timedelta(days=14)),
    ('3 days ago', datetime.now() - timedelta(days=3)),
    ('4 days ago', datetime.now() - timedelta(days=4)),
    ('4daysago', datetime.now() - timedelta(days=4)),
    ('2 weeks ago', datetime.now() - timedelta(days=14)),
    ('3 weeks ago', datetime.now() - timedelta(days=21)),
    ('3 wks ago', datetime.now() - timedelta(days=21)),
    ('3 w ago', datetime.now() - timedelta(days=21)),
    ('3 wago', datetime.now() - timedelta(days=21)),
    ('3wago', datetime.now() - timedelta(days=21)),
    ('3wks ago', datetime.now() - timedelta(days=21)),
    ('12/8/24', datetime.strptime("12/08/24", "%d/%m/%y")),
    ('3/8/2024', datetime.strptime("3/08/24", "%d/%m/%y")),
])
def test_parse_date_entry(date_entry, intended_date):
    """
    The script, when inputting time entries, must parse the following
    strings into start and end dates
    - yesterday
    - today
    - last week
    - last fortnight
    - N days ago
    - N weeks ago
    - DD/M/YY
    - DD/MM/YYYY
    """
    intended_date = intended_date.replace(hour=0, minute=0, second=0, microsecond=0)
    time_entry = TimeEntry(date_entry, "an hour ago")
    parsed_date = time_entry.parse_date_string(date_entry)

    assert parsed_date == intended_date, f"{parsed_date} from {date_entry} doesnt match {intended_date}"

@pytest.mark.parametrize("time_string, intended_time", [
    ("an hour ago", (datetime.now() - timedelta(hours=1)).replace(second=0,microsecond=0)),
    ("noon", datetime.now().replace(hour=12, minute=0, second=0, microsecond=0)),
    ("lunch", datetime.now().replace(hour=12, minute=0, second=0, microsecond=0)),
    ("lunch time", datetime.now().replace(hour=12, minute=0, second=0, microsecond=0)),
    ("midnight", datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)),
    ("now", datetime.now().replace(second=0, microsecond=0)),
    ("1hr ago", (datetime.now().replace(second=0, microsecond=0) - timedelta(hours=1))),
    ("1 hr ago", (datetime.now().replace(second=0, microsecond=0) - timedelta(hours=1))),
    ("2hrs ago", (datetime.now().replace(second=0, microsecond=0) - timedelta(hours=2))),
    ("2 hrs ago", (datetime.now().replace(second=0, microsecond=0) - timedelta(hours=2))),
    ("13:45", datetime.now().replace(hour=13, minute=45, second=0, microsecond=0)),
    ("1345", datetime.now().replace(hour=13, minute=45, second=0, microsecond=0)),
    ("01:00 PM", datetime.now().replace(hour=13, minute=0, second=0, microsecond=0)),
    ("01:00PM", datetime.now().replace(hour=13, minute=0, second=0, microsecond=0)),
    ("01:00 AM", datetime.now().replace(hour=1, minute=0, second=0, microsecond=0)),
    ("01:00AM", datetime.now().replace(hour=1, minute=0, second=0, microsecond=0)),
    ("1 pM", datetime.now().replace(hour=13, minute=0, second=0, microsecond=0)),
    ("1 pm", datetime.now().replace(hour=13, minute=0, second=0, microsecond=0)),
    ("1 Am", datetime.now().replace(hour=1, minute=0, second=0, microsecond=0)),
    ("1 am", datetime.now().replace(hour=1, minute=0, second=0, microsecond=0)),
    ("10 am", datetime.now().replace(hour=10, minute=0, second=0, microsecond=0)),
    ("10 pm", datetime.now().replace(hour=22, minute=0, second=0, microsecond=0)),
    ])
def test_parse_time_string_to_seconds_into_day(time_string, intended_time):
    """
    The script, when inputting time entries, must parse the following
    tarm strings into the start and end times

    - an hour ago
    - noon
    - lunch
    - lunch time
    - midnight
    - now

    - 1hr ago
    - 1 hr ago
    - 2hrs ago
    - 2 hrs ago

    - HH:MM
    - HHMM

    - HH:MM AM
    - HH:MMAM

    - HH:MM PM
    - HH:MMPM

    - HH PM
    """
    now = datetime.now().replace(hour=0,minute=0, second=0, microsecond=0)
    time_entry = TimeEntry("1/1/2024", "an hour ago")
    parsed_time = time_entry.parse_time_string(time_string)

    assert parsed_time == intended_time - now, f"{time_string} parsed as {parsed_time} is not equal to {intended_time - now}"

@pytest.mark.parametrize("duration_string, intended_seconds", [
    ("a second", 1),
    ("10 seconds", 10),
    ("10 sec", 10),
    ("10 s", 10),
    ("a minute", 60),
    ("5 minutes", 300),
    ("5 min", 300),
    ("5 m", 300),
    ("an hour", 3600),
    ("anhour", 3600),
    ("3 hours", 10800),
    ("3 hr", 10800),
    ("3 h", 10800),
    ("3h", 10800),
    ("a day", 86400),
    ("2 days", 172800),
    ("2 d", 172800),
    ("2 day", 172800),
    ("1.5 hours", 5400),
    ("1.5 hr", 5400),
    ("1.5 h", 5400),
    ("0.5 minutes", 30),
    ("1 day 2 hours 3 minutes 4 seconds", 93784),
    ("1d 2h 3m 4s", 93784),
    ("1.25 days", 108000),
    ("0.75 hours", 2700),
    ("1 day and 2 hours", 93600),
    ("a day and an hour", 90000),
    ("a day and an hour and 34.5mins + 33s", 90000 + 2103),
    ("1day+1hr+34.5m+33s", 90000 + 2103),
    ])
def test_parse_duration_to_seconds(duration_string, intended_seconds):
    """
    The script, when inputting a duration string, must convert the
    duriation into seconds and return it. Durations may be integers OR
    float values.

    seconds handlers
    - a(n) second
    - N s(ec)(ond)(s) or Ns(ec)(ond)(s)

    minutes handlers
    - a min(ute)
    - N min(ute)(s) or Nmin(ute)(s)

    hours handlers
    - a(n) hour(s)
    - N h(ou)r(s) Nh(our)r(s)

    days handler
    - a(n) day(s)
    - N d(ay)(s) or Nd(ay)(s)

    """
    time_entry = TimeEntry("yesterday", "lunch")
    parsed_seconds = time_entry.parse_duration_string(duration_string)
    assert parsed_seconds == intended_seconds, f"Expected {intended_seconds}, but got {parsed_seconds}"

@pytest.mark.parametrize("start_date, start_time, end_date, end_time, intended_duration_sec", [
    ("yesterday","lunch", "today", "lunch", 2*43200),
    ("yesterday","lunch", "today", "midnight", 43200),
    ])

def test_calculate_duration_to_seconds(start_date, start_time, end_date, end_time, intended_duration_sec):
    time_entry = TimeEntry(start_date, start_time)
    time_entry.set_end_ts(end_date, end_time)
    duration_sec = time_entry.duration_sec
    assert duration_sec == intended_duration_sec, f"{duration_sec}s does not equal intended {intended_duration_sec}s"

@pytest.mark.parametrize("start_date, start_time, duration_override,intended_end_ts", [
    ("12/12/2024","lunch", "3hrs", datetime.strptime("12-12-2024_15:00", "%d-%m-%Y_%H:%M")),
    ("13/01/2024","13:00", "an hour and 30 minutes", datetime.strptime("13-01-2024_14:30", "%d-%m-%Y_%H:%M")),
    ("13/01/2024","1pm", "an hour and 30 minutes", datetime.strptime("13-01-2024_14:30", "%d-%m-%Y_%H:%M")),
    ])
def test_end_ts_with_duration(start_date, start_time, duration_override, intended_end_ts):
    time_entry = TimeEntry(start_date, start_time)
    time_entry.set_duration(duration_override)
    assert time_entry.end_ts == intended_end_ts, f"{time_entry.end_ts} is not equal to intended {intended_end_ts} after {duration_override} was applied"

@pytest.mark.parametrize("method", ["note", "client", "category"])
@pytest.mark.parametrize("text_to_write", ["this is a note", '\this is a Note', "\rdigism\n\r\t"])
def test_details_set_entry(method, text_to_write):
    """
    The module must strip the text then write it to the text_to_write.
    """
    time_entry = TimeEntry("today", "lunch")
    getattr(time_entry, f"set_{method}")(text_to_write)
    property_value = getattr(time_entry, method)
    assert property_value == text_to_write.strip(), f"{property_value} not equal to the intended {text_to_write}"

@pytest.mark.parametrize("method", ["note", "client", "category"])
@pytest.mark.parametrize("text_to_write", ["this is a note", '\this is a Note', "\rdigism\n\r\t"])
def test_details_init_entry(method, text_to_write):
    """
    The module, when initializing text details, must strip and save the text into the property...
    """
    time_entry = TimeEntry("today", "lunch", **{method: text_to_write})
    property_value = getattr(time_entry, method)
    assert property_value == text_to_write.strip(), f"{property_value} not equal to the intended {text_to_write}"

def test_duration_override():
    """
    When overrideing the duration when a time is set, the new duration
    must be the duratino override. The end_ts must be updated
    accordingly
    """
    time_entry = TimeEntry("yesterday","lunch")
    time_entry.set_end_ts("today", "lunch")
    time_entry.set_duration("1hrs")
    assert time_entry.duration_sec == 3600, "The duration was not overridden..."

def test_end_ts_override():
    """
    When setting the end_ts, the duration properpty must output the
    updated end_ts duration NOT the old duration value...
    """
    time_entry = TimeEntry("yesterday", "lunch")
    time_entry.set_duration("1hr")
    time_entry.set_end_ts("today", "midnight")

    duration_hrs = time_entry.duration_sec / 3600

    if (duration_hrs != 12):

        # Check this fails for this implementation...
        if (time_entry.duration_sec_override):
            pytest.fail("time_entry.end_ts didnt reset the time_entry.duration_sec_override property...")

        pytest.fail(f"duration of {duration_hrs} isnt 12hrs")

    pass
